import { Expectation } from '@serenity-js/core';
/**
 * Creates an [expectation](https://serenity-js.org/api/core/class/Expectation/) that is met when the actual value of type `number`
 * is greater than the expected `number`.
 *
 * ## Ensuring that a given number is greater than the expected number
 *
 * ```ts
 * import { actorCalled } from '@serenity-js/core'
 * import { Ensure, isGreaterThan } from '@serenity-js/assertions'
 *
 * await actorCalled('Ester').attemptsTo(
 *   Ensure.that(10, isGreaterThan(5)),
 * )
 * ```
 *
 * ## Ensuring that a given number is within the expected range
 *
 * ```ts
 * import { actorCalled, Expectation, d } from '@serenity-js/core'
 * import { Ensure, and, equals, isGreaterThan, isLessThan, or } from '@serenity-js/assertions'
 *
 * const isWithinRange = (lowerBound: Answerable<number>, upperBound: Answerable<number>) =>
 *   Expectation.to(d`have value that is between ${ lowerBound } and ${ upperBound }`)
 *     .soThatActual(
 *       and(
 *         or(equals(lowerBound), isGreaterThan(lowerBound)),
 *         or(equals(upperBound), isLessThan(upperBound)),
 *       )
 *     ),
 *
 * await actorCalled('Ester').attemptsTo(
 *   Ensure.that(
 *     7,
 *     isWithinRange(5, 10)
 *   ),
 * )
 * ```
 *
 * @param expected
 *
 * @group Expectations
 */
export declare const isGreaterThan: (expected: import("@serenity-js/core").Answerable<number>) => Expectation<number>;
//# sourceMappingURL=isGreaterThan.d.ts.map